import PrimitiveRenderer from "./PrimitiveRenderer.js";

export default class CubeRenderer extends PrimitiveRenderer {
    constructor(size=1.0, faceColors=null) {
        super();
        this.initCube(size, faceColors);
    }

    initCube(size, faceColors) {
        const offset = size / 2;

        this.vertices = [
            // Front face
            -offset, -offset, offset,
            offset, -offset, offset,
            offset, offset, offset,
            -offset, offset, offset,
            // Back face
            -offset, -offset, -offset,
            -offset, offset, -offset,
            offset, offset, -offset,
            offset, -offset, -offset,
            // Top face
            -offset, offset, -offset,
            -offset, offset, offset,
            offset, offset, offset,
            offset, offset, -offset,
            // Bottom face
            -offset, -offset, -offset,
            offset, -offset, -offset,
            offset, -offset, offset,
            -offset, -offset, offset,
            // Right face
            offset, -offset, -offset,
            offset, offset, -offset,
            offset, offset, offset,
            offset, -offset, offset,
            // Left face
            -offset, -offset, -offset,
            -offset, -offset, offset,
            -offset, offset, offset,
            -offset, offset, -offset,
        ];

        if (!faceColors) {
            faceColors = [
                [1.0, 1.0, 1.0, 1.0], // Front face: white
                [1.0, 0.0, 0.0, 1.0], // Back face: red
                [0.0, 1.0, 0.0, 1.0], // Top face: green
                [0.0, 0.0, 1.0, 1.0], // Bottom face: blue
                [1.0, 1.0, 0.0, 1.0], // Right face: yellow
                [1.0, 0.0, 1.0, 1.0], // Left face: purple
            ];
        }

        // Convert the array of colors into a table for all the vertices.
        this.colors = [];
        for (var j = 0; j < faceColors.length; ++j) {
            const c = faceColors[j];
            // Repeat each color four times for the four vertices of the face
            this.colors = this.colors.concat(c, c, c, c);
        }

        this.indices = [
            0, 1, 2, 0, 2, 3, // front
            4, 5, 6, 4, 6, 7, // back
            8, 9, 10, 8, 10, 11, // top
            12, 13, 14, 12, 14, 15, // bottom
            16, 17, 18, 16, 18, 19, // right
            20, 21, 22, 20, 22, 23, // left
        ];
    }
}