import PrimitiveRenderer from "./PrimitiveRenderer.js";

export default class CylinderRenderer extends PrimitiveRenderer {
    constructor(height=2.0, radius=0.5, segmentCount=100, color=[1.0, 1.0, 1.0 , 1.0]) {
        super();
        this.initShape(height, radius, segmentCount, color);
    }

    initShape(height, radius, segmentCount, color) {
        this.vertices = [];

        // set up the top and bottom edge
        const angleStep = 2 * Math.PI / segmentCount;
        for (let theta = 0; theta < 2 * Math.PI; theta += angleStep) {
            const x = radius * Math.cos(theta);
            const z = radius * Math.sin(theta);
            this.vertices.push(
                x, 0, z,
                x, height, z
            );
        }

        // add the top and bottom center vertices
        this.vertices.push(
            // bottom center
            0, 0, 0,
            // top center
            0, height, 0
        );

        // fill in indices
        this.indices = [];
        for (let i = 0; i < segmentCount * 2; i += 2) {
            // get the indices for this vertex, the one to the right and the corresponding top ones
            const cib = i;
            const cit = (cib + 1) % (segmentCount * 2);

            const nib = (i + 2) % (segmentCount * 2);
            const nit = (nib + 1) % (segmentCount * 2);
            

            this.indices.push(
                // bottom face
                segmentCount * 2, nib, cib,

                // outer face (2 tris)
                cib, nib, cit,
                nib, nit, cit,

                // TOP face
                segmentCount * 2 + 1, cit, nit
            );
        }
        
        // fill colors
        this.colors =  [];
        for (let i = 0; i < this.vertices.length / 3; i++) {
            this.colors.push(...color);
        }
    }
}