import PrimitiveRenderer from "./PrimitiveRenderer.js";
import {
    buildSphereVertices, buildSphereIndices
} from "../util/SphereUtil.js";

export default class SphereRenderer extends PrimitiveRenderer {
    constructor(radius, color, verticalDivisions = 100, horizontalDivisions = 20) {
        super();
        this.initShape(radius, color, verticalDivisions, horizontalDivisions);
    }

    /**
     * Initializes the shape. We use a UV sphere since it is the easiest to set up
     */
    initShape(radius, color, verticalDivisions, horizontalDivisions) {
        // initialize the vertices for a UV sphere
        this.vertices = buildSphereVertices(radius, verticalDivisions, horizontalDivisions, 0, horizontalDivisions);

        // number of vertices in sphere (expect for top and bottom)
        const sphereVertexCount = this.vertices.length / 3 - 2;

        // set up the index array for the tris
        // our actual number horizontal divisions (since we skip it at 0 and 2PI)
        this.indices = buildSphereIndices(sphereVertexCount, verticalDivisions, horizontalDivisions, 0, horizontalDivisions);

        // fill colors
        this.colors = [];
        for (let i = 0; i < this.vertices.length / 3; i++) {
            this.colors.push(...color);
        }
    }
}